#****************************************************************************
#         The scope of use of the material contained herein is limited
#         strictly to the terms of the agreement which grants permission
#         for the use of the material.  Distribution of this material
#            and all copies thereof to any entity other than the one
#           mentioned in the agreement, as the term is defined in the
#        agreement, is strictly forbidden without a prior written consent.
#****************************************************************************
#****************************************************************************
# 
#  Code generator script - lfsr logic
#
#  Filename        :  lfsr-gen.py
#  Revision        :  $Revision: 1.0 $
#  Checkin Date    :  $Date: 2008/05/28 12:29:59 $
#  Last checkin by :  $Author: teleistad $
#  SVN State       :  $State: Exp $
#
#  Project         :  Master Thesis
#
#  Author          :  <Tor Erik Leistad>
#  Date            :  <02.2008>
#
#****************************************************************************

from sys import stdin

#global vars
bitwidth = 0
polynom = ""

print ""
print "LFSR-generator"
print ""
print "Reads info from configuration file and generate an LFSR of given length"
print ""

#files
confFile = open('../../../rtl/module/chip/include/chip_params.v', 'r')
tableFile = open('lfsrtable.txt')
lfsrFile = open('../../../rtl/module/lfsr/custom/verilog/lfsr.v', 'w')

#pars conf file
line = confFile.readline()
linecmd = line.partition(" ")

while line : 
    if linecmd[0][0:1] == "//" or linecmd[0] == "//\r\n" or linecmd[0] == "//\n":
        line = confFile.readline()
        linecmd = line.partition(" ")
    if linecmd[0] == "parameter" :
        linevar = linecmd[2].partition(" ")
        if linevar[0] == "LFSR_BITS" :
            var = linevar[2].partition("=")
            var = var[2].partition(";")
            bitwidth = int(var[0])
    line = confFile.readline()
    linecmd = line.partition(" ")
    
print "bitwidth is " + str(bitwidth)

#lookup lfsrtable file
print "trying to find polynom for " + str(bitwidth) + " bits"
line = tableFile.readline()
linecmd = line.partition(" ")
while line :
    if linecmd[0] == "#" or linecmd[0] == "#\n" :
        line = tableFile.readline()
        linecmd = line.partition(" ")
    if linecmd[0] == str(bitwidth) :
        polynom = linecmd[2]
        break
    else :
        line = tableFile.readline()
        linecmd = line.partition(" ")
        
print "feedback into registers #: " + polynom

confFile.close()
tableFile.close()


#write output file
print "generating verilog file for lfsr"
lfsrFile.write("//this module is autogenerated, do not edit!\n")
lfsrFile.write("module lfsr(\n")
lfsrFile.write("    //outputs\n")
lfsrFile.write("    lfsr_out,\n")
lfsrFile.write("    lfsr_so,\n")
lfsrFile.write("    //inputs\n")
lfsrFile.write("    lfsr_clk,\n")
lfsrFile.write("    lfsr_rst_n,\n")
lfsrFile.write("    lfsr_en,\n")
lfsrFile.write("    lfsr_load,\n")
lfsrFile.write("    lfsr_si\n")
lfsrFile.write("    );\n")
lfsrFile.write("\n")
lfsrFile.write("    //outputs\n")
lfsrFile.write("    output lfsr_so;\n")
lfsrFile.write("    output ["+str(bitwidth-1)+":0] lfsr_out;\n")
lfsrFile.write("    //inputs\n")
lfsrFile.write("    input lfsr_clk;\n")
lfsrFile.write("    input lfsr_rst_n;\n")
lfsrFile.write("    input lfsr_en;\n")
lfsrFile.write("    input lfsr_load;\n")
lfsrFile.write("    input lfsr_si;\n")
lfsrFile.write("    \n")
lfsrFile.write("    reg ["+str(bitwidth-1)+":0] lfsr_value;\n")
lfsrFile.write("    \n")
lfsrFile.write("    assign lfsr_so = lfsr_value[0];\n")
lfsrFile.write("    assign lfsr_out = lfsr_value;\n")
lfsrFile.write("    \n")
lfsrFile.write("    always @ (posedge lfsr_clk or negedge lfsr_rst_n)\n")
lfsrFile.write("    begin\n")
lfsrFile.write("        if (lfsr_rst_n == 'b0)\n")
lfsrFile.write("            lfsr_value <= 'b1;\n")
lfsrFile.write("        else\n")
lfsrFile.write("            begin\n")
lfsrFile.write("                if (lfsr_load == 1'b1)\n")
lfsrFile.write("                    lfsr_value <= {lfsr_si,lfsr_value["+str(bitwidth-1)+":1]};\n")
lfsrFile.write("                else\n")
lfsrFile.write("                    if (lfsr_en == 1'b1)\n")
lfsrFile.write("                        begin\n")
lfsrFile.write("                            //lfsr feedback connections\n")

#generats the internal feedback logic
feedback_reg = polynom.partition(",")

lfsrFile.write("                            lfsr_value["+str(bitwidth-1)+"] <= lfsr_value[0];\n")

feedback_reg = feedback_reg[2].partition(",")

for i in range(1,bitwidth):
    if i == int(feedback_reg[0]) :
        lfsrFile.write("                            lfsr_value["+str(bitwidth-1-i)+"] <= lfsr_value["+str(bitwidth-i)+"] ^ lfsr_value[0];\n")
        feedback_reg = feedback_reg[2].partition(",")
    else :
        lfsrFile.write("                            lfsr_value["+str(bitwidth-1-i)+"] <= lfsr_value["+str(bitwidth-i)+"];\n")

lfsrFile.write("                        end\n")
lfsrFile.write("                    else\n")
lfsrFile.write("                        lfsr_value <= lfsr_value;\n")
lfsrFile.write("            end\n")
lfsrFile.write("    end\n")
lfsrFile.write("    \n")
lfsrFile.write("endmodule //lfsr\n")
lfsrFile.write("\n")



#cleanup
lfsrFile.close()
