#****************************************************************************
#         The scope of use of the material contained herein is limited
#         strictly to the terms of the agreement which grants permission
#         for the use of the material.  Distribution of this material
#            and all copies thereof to any entity other than the one
#           mentioned in the agreement, as the term is defined in the
#        agreement, is strictly forbidden without a prior written consent.
#****************************************************************************
#****************************************************************************
# 
#  Code generator script - misr logic
#
#  Filename        :  misr-gen.py
#  Revision        :  $Revision: 1.0 $
#  Checkin Date    :  $Date: 2008/05/28 12:29:59 $
#  Last checkin by :  $Author: teleistad $
#  SVN State       :  $State: Exp $
#
#  Project         :  Master Thesis
#
#  Author          :  <Tor Erik Leistad>
#  Date            :  <02.2008>
#
#****************************************************************************

from sys import stdin

#global vars
bitwidth = 0
polynom = ""

print ""
print "MISR-generator"
print ""
print "Reads info from configuration file and generate an MISR of given length"
print ""

#files
confFile = open('../../../rtl/module/chip/include/chip_params.v', 'r')
tableFile = open('../lfsr-gen/lfsrtable.txt')
misrFile = open('../../../rtl/module/misr/custom/verilog/misr.v', 'w')


#pars conf file
line = confFile.readline()
linecmd = line.partition(" ")

while line : 
    if linecmd[0][0:1] == "//" or linecmd[0] == "//\r\n" or linecmd[0] == "//\n":
        line = confFile.readline()
        linecmd = line.partition(" ")
    if linecmd[0] == "parameter" :
        linevar = linecmd[2].partition(" ")
        if linevar[0] == "MISR_BITS" :
            var = linevar[2].partition("=")
            var = var[2].partition(";")
            bitwidth = int(var[0])
    line = confFile.readline()
    linecmd = line.partition(" ")
    
    
print "bitwidth is " + str(bitwidth)

#lookup lfsrtable file
print "trying to find polynom for " + str(bitwidth) + " bits"
line = tableFile.readline()
linecmd = line.partition(" ")
while line :
    if linecmd[0] == "#" or linecmd[0] == "#\n" :
        line = tableFile.readline()
        linecmd = line.partition(" ")
    if linecmd[0] == str(bitwidth) :
        polynom = linecmd[2]
        break
    else :
        line = tableFile.readline()
        linecmd = line.partition(" ")
        
print "feedback into registers #: " + polynom

confFile.close()
tableFile.close()


#write output file
print "generating verilog file for misr"
misrFile.write("//this module is autogenerated, do not edit!\n")
misrFile.write("module misr(\n")
misrFile.write("    //outputs\n")
misrFile.write("    misr_so,\n")
misrFile.write("    //inputs\n")
misrFile.write("    misr_clk,\n")
misrFile.write("    misr_rst_n,\n")
misrFile.write("    misr_en,\n")
misrFile.write("    misr_load,\n")
misrFile.write("    misr_si,\n")
misrFile.write("    misr_in\n")
misrFile.write("    );\n")
misrFile.write("\n")
misrFile.write("    //outputs\n")
misrFile.write("    output misr_so;\n")
misrFile.write("    //inputs\n")
misrFile.write("    input misr_clk;\n")
misrFile.write("    input misr_rst_n;\n")
misrFile.write("    input misr_en;\n")
misrFile.write("    input misr_load;\n")
misrFile.write("    input misr_si;\n")
misrFile.write("    input ["+str(bitwidth-1)+":0] misr_in;\n")
misrFile.write("    \n")
misrFile.write("    reg ["+str(bitwidth-1)+":0] misr_value;\n")
misrFile.write("    \n")
misrFile.write("    assign misr_so = misr_value[0];\n")
misrFile.write("    \n")
misrFile.write("    always @ (posedge misr_clk or negedge misr_rst_n)\n")
misrFile.write("    begin\n")
misrFile.write("        if (misr_rst_n == 'b0)\n")
misrFile.write("            misr_value <= 'b0;\n")
misrFile.write("        else\n")
misrFile.write("            begin\n")
misrFile.write("                if (misr_load == 1'b1)\n")
misrFile.write("                    misr_value <= {misr_si,misr_value["+str(bitwidth-1)+":1]};\n")
misrFile.write("                else\n")
misrFile.write("                    if (misr_en == 1'b1)\n")
misrFile.write("                        begin\n")
misrFile.write("                            //misr feedback connections\n")

#generats the internal feedback logic
feedback_reg = polynom.partition(",")

misrFile.write("                            misr_value["+str(bitwidth-1)+"] <= misr_value[0] ^ misr_in[0];\n")

feedback_reg = feedback_reg[2].partition(",")

for i in range(1,bitwidth):
    if i == int(feedback_reg[0]) :
        misrFile.write("                            misr_value["+str(bitwidth-1-i)+"] <= misr_value["+str(bitwidth-i)+"] ^ misr_value[0] ^ misr_in["+str(i)+"];\n")
        feedback_reg = feedback_reg[2].partition(",")
    else :
        misrFile.write("                            misr_value["+str(bitwidth-1-i)+"] <= misr_value["+str(bitwidth-i)+"] ^ misr_in["+str(i)+"];\n")

misrFile.write("                        end\n")
misrFile.write("                    else\n")
misrFile.write("                        misr_value <= misr_value;\n")
misrFile.write("            end\n")
misrFile.write("    end\n")
misrFile.write("    \n")
misrFile.write("endmodule //misr\n")
misrFile.write("\n")



#cleanup
misrFile.close()
