#****************************************************************************
#         The scope of use of the material contained herein is limited
#         strictly to the terms of the agreement which grants permission
#         for the use of the material.  Distribution of this material
#            and all copies thereof to any entity other than the one
#           mentioned in the agreement, as the term is defined in the
#        agreement, is strictly forbidden without a prior written consent.
#****************************************************************************
#****************************************************************************
# 
#  Code generator script - parameter checking script
#
#  Filename        :  parameter_check.py
#  Revision        :  $Revision: 1.0 $
#  Checkin Date    :  $Date: 2008/05/28 12:29:59 $
#  Last checkin by :  $Author: teleistad $
#  SVN State       :  $State: Exp $
#
#  Project         :  Master Thesis
#
#  Author          :  <Tor Erik Leistad>
#  Date            :  <02.2008>
#
#****************************************************************************

from sys import stdin
import math

#return the greates common divisor for a and b
def gcd(a, b):

    if (b == 0):
        return a
    return gcd(b, a % b)


#find the maximum distinct number fo scan patterns given some parameters
def maxpat(lfsr_size, len_scan_chain):

    num_pat = 2**lfsr_size-1 #patterns generated by lfsr
    num_scan_pat = 0         #num of patterns in scan chains
    num_rem = 0              #reminder 

    while(1):
        if(num_rem == len_scan_chain):
            num_scan_pat = num_scan_pat + 1
            break
        elif(num_rem>len_scan_chain):
            num_rem = num_rem-len_scan_chain;
            num_scan_pat = num_scan_pat + 1
        else:
            num_rem = num_pat - (len_scan_chain-num_rem)
            num_scan_pat = num_scan_pat + 1

    return num_scan_pat


#given a size for the lfsr, find the number of shifts required to maximise the 
#total number of patterns
def findshifts(lfsr_size, len_scan_chain):
    
    num_pat = 2**lfsr_size-1
    shifts = len_scan_chain
    
    while(gcd(shifts,num_pat) != 1):
        print "shifts=" +str(shifts)+" ; num_scan_pat="+str(maxpat(lfsr_size,shifts))
        shifts = shifts + 1
    print "shifts=" +str(shifts)+" ; num_scan_pat="+str(maxpat(lfsr_size,shifts))
    return shifts


###                   main                   ###
### claculations require maximum length lfsr ###

# the parameters
lfsr_size = 8
len_scan_chains = 24
num_scan_chains = 16
num_scan_patterns = 10

#read from conf file
confFile = open('../../../rtl/module/chip/include/chip_params.v', 'r')

#parse conf file
line = confFile.readline()
linecmd = line.partition(" ")

while line : 
    if linecmd[0][0:1] == "//" or linecmd[0] == "//\r\n" or linecmd[0] == "//\n":
        line = confFile.readline()
        linecmd = line.partition(" ")
    if linecmd[0] == "parameter" :
        linevar = linecmd[2].partition(" ")
        if linevar[0] == "NUM_SCAN_CHAINS" :
            var = linevar[2].partition("=")
            var = var[2].partition(";")
            num_scan_chains = int(var[0])
        if linevar[0] == "LEN_SCAN_CHAINS" :
            var = linevar[2].partition("=")
            var = var[2].partition(";")
            len_scan_chains = int(var[0])
        if linevar[0] == "LFSR_BITS" :
            var = linevar[2].partition("=")
            var = var[2].partition(";")
            lfsr_size = int(var[0])
    line = confFile.readline()
    linecmd = line.partition(" ")

confFile.close()

#for best results with the given lfsr, find optimal numbers of shifts
opt_shifts = findshifts(lfsr_size,len_scan_chains)
#the maximum number of patterns, 
opt_pat = maxpat(lfsr_size,opt_shifts)


#check for minimum lfsr size
print ""
if ((2**lfsr_size-1) > (len_scan_chains*num_scan_chains)):
    print "LFSR+PS satisfies 2**n-1 > Nff*Nsc"
else:
    print "LFSR of size n="+str(lfsr_size)+" does not satisfy 2**n-1 > Nff*Nsc"
    print "  min. size of lfsr is: "+str(int(math.ceil(math.log(len_scan_chains*num_scan_chains,2))))
   
if ((2**lfsr_size-1) > (len_scan_chains*num_scan_patterns)):
    print "LFSR satisfies 2**n-1 > Nff*Ntv"
else:
    print "LFSR of size n="+str(lfsr_size)+" does not satisfy 2**n-1 > Nff*Ntv"   
    print "  min. size of lfsr is: "+str(int(math.ceil(math.log(len_scan_chains*num_scan_patterns,2))))
 
print ""
print "For LFSR of size "+str(lfsr_size)+", and scan chains of length "+str(len_scan_chains)
print "the optimal number of shifts are: "+str(opt_shifts)
print "which gives "+str(opt_pat)+" scan patterns."
print ""
