#****************************************************************************
#         The scope of use of the material contained herein is limited
#         strictly to the terms of the agreement which grants permission
#         for the use of the material.  Distribution of this material
#            and all copies thereof to any entity other than the one
#           mentioned in the agreement, as the term is defined in the
#        agreement, is strictly forbidden without a prior written consent.
#****************************************************************************
#****************************************************************************
# 
#  Code generator script - phase shifter logic
#
#  Filename        :  ps-gen.py
#  Revision        :  $Revision: 1.0 $
#  Checkin Date    :  $Date: 2008/05/28 12:29:59 $
#  Last checkin by :  $Author: teleistad $
#  SVN State       :  $State: Exp $
#
#  Project         :  Master Thesis
#
#  Author          :  <Tor Erik Leistad>
#  Date            :  <02.2008>
#
#****************************************************************************

from sys import stdin

#limitations, max number of feedback terms is 7... add more if-options for more...

print ""
print "Phase Shift-generator"
print ""
print "Reads info from configuration file and generate an Phase Shifter"
print "of with given number of inputs, outputs and shift"
print ""

#variables that will be read from conf files (added some test values)
lfsrBits = 32
PSoutputs = 32
minshift = 1000
maxones = 4
feedbackterms = 4

#read configurationfiles
#files
confFile = open('../../../rtl/module/chip/include/chip_params.v', 'r')
tableFile = open('../lfsr-gen/lfsrtable.txt')
PSFile = open('../../../rtl/module/ps/custom/verilog/phaseshifter.v', 'w')

#pars conf file
line = confFile.readline()
linecmd = line.partition(" ")

while line : 
    if linecmd[0][0:1] == "//" or linecmd[0] == "//\r\n" or linecmd[0] == "//\n":
        line = confFile.readline()
        linecmd = line.partition(" ")
    if linecmd[0] == "parameter" :
        linevar = linecmd[2].partition(" ")
        if linevar[0] == "LFSR_BITS" :
            var = linevar[2].partition("=")
            var = var[2].partition(";")
            lfsrBits = int(var[0])
        if linevar[0] == "PS_OUTPUTS" :
            var = linevar[2].partition("=")
            var = var[2].partition(";")
            PSoutputs = int(var[0])
        if linevar[0] == "PS_MINSHIFT" :
            var = linevar[2].partition("=")
            var = var[2].partition(";")
            minshift = int(var[0])
        if linevar[0] == "PS_MAXONES" :
            var = linevar[2].partition("=")
            var = var[2].partition(";")
            maxones = int(var[0])
    line = confFile.readline()
    linecmd = line.partition(" ")


print "Input bitwidth is " + str(lfsrBits)
print "Output bitwidth is " + str(PSoutputs)
print "Min. shift between scan chains is " + str(minshift)
print "Maximum of ones in vector is " + str(maxones)

#lookup lfsrtable file
print "trying to find polynom for " + str(lfsrBits) + " bits"
line = tableFile.readline()
linecmd = line.partition(" ")
while line :
    if linecmd[0] == "#" or linecmd[0] == "#\n" :
        line = tableFile.readline()
        linecmd = line.partition(" ")
    if linecmd[0] == str(lfsrBits) :
        polynom = linecmd[2]
        break
    else :
        line = tableFile.readline()
        linecmd = line.partition(" ")
        
#convert string polynom to array
feedback_list = []
feedback_reg = polynom.partition(",")
while feedback_reg[0] :
    feedback_list.append(int(feedback_reg[0]))
    feedback_reg = feedback_reg[2].partition(",")

print "feedback polynom : " + str(feedback_list)


feedbackterms = len(feedback_list)-1
print "feedback terms: " + str(feedbackterms)


confFile.close()
tableFile.close()

#check that the parameters dont conflict:


#generat and init lfsr array
lfsr = []
lfsr.append(int(1))
for i in range (1,lfsrBits):
    lfsr.append(int(0))

#variables
shiftcount = 0
shiftcounttotal = 0
onescount = 0
outputcount = 0
feedback = 0

resultvectors = []

#defining the feedback calculation here saves some runtime as the overhead of running
#the function is less than checking all the if's...
fl = []
fl.extend(feedback_list)
fl.pop(0)
if feedbackterms == 2:
    feedbackcalc = lambda : lfsr[fl[0]-1] ^ lfsr[fl[1]-1]
if feedbackterms == 3:
    feedbackcalc = lambda : lfsr[fl[0]-1] ^ lfsr[fl[1]-1] ^ lfsr[fl[2]-1]
if feedbackterms == 4:
    feedbackcalc = lambda : lfsr[fl[0]-1] ^ lfsr[fl[1]-1] ^ lfsr[fl[2]-1] ^ lfsr[fl[3]-1]
if feedbackterms == 5:
    feedbackcalc = lambda : lfsr[fl[0]-1] ^ lfsr[fl[1]-1] ^ lfsr[fl[2]-1] ^ lfsr[fl[3]-1] ^ lfsr[fl[4]-1]
if feedbackterms == 6:
    feedbackcalc = lambda : lfsr[fl[0]-1] ^ lfsr[fl[1]-1] ^ lfsr[fl[2]-1] ^ lfsr[fl[3]-1] ^ lfsr[fl[4]-1] ^ lfsr[fl[5]-1]
if feedbackterms == 7:
    feedbackcalc = lambda : lfsr[fl[0]-1] ^ lfsr[fl[1]-1] ^ lfsr[fl[2]-1] ^ lfsr[fl[3]-1] ^ lfsr[fl[4]-1] ^ lfsr[fl[5]-1] ^ lfsr[fl[6]-1]

while outputcount < PSoutputs:

    lfsr.insert(0,feedbackcalc())
    lfsr.pop()
    
    shiftcount = shiftcount + 1
    shiftcounttotal = shiftcounttotal + 1
    
    #just to indicate how fast it is working...
    if shiftcounttotal % 100000 == 0:
        print shiftcounttotal
    
    if shiftcount > minshift:
        onescount = sum(lfsr)
        
        if onescount <= 4:
            resultvectors.append([])
            resultvectors[len(resultvectors)-1].extend(lfsr)
            shiftcount = 0
            outputcount = outputcount + 1
        onescount = 0
    
    #this is only added to stop simulation for time measurment for 
    #algorithm optimisation , simulate 100000 patterns
    #time pyhton ps-gen.py
    #if shiftcounttotal == 100000:
    #    break

print " "
for i in range(0,len(resultvectors)):
    print resultvectors[i]

print "lfsr cycles :" +str(shiftcounttotal)

#generate verilog:
PSFile.write("//Autogenerated file - do not edit\n\n")
PSFile.write("module phaseshifter(\n")
PSFile.write("    //outputs\n")
PSFile.write("    phaseshifter_out,\n")
PSFile.write("    //inputs\n")
PSFile.write("    phaseshifter_in\n")
PSFile.write("    );\n")
PSFile.write("\n")
PSFile.write("    output ["+str(PSoutputs-1)+":0] phaseshifter_out;\n")
PSFile.write("    input  ["+str(lfsrBits-1)+ ":0] phaseshifter_in;\n")
PSFile.write("\n")
PSFile.write("\n")
for i in range(0,len(resultvectors)):
    PSFile.write("    assign phaseshifter_out["+str(i)+"] = ")
    xorbits = sum(resultvectors[i])
    added = 0
    for j in range(0,lfsrBits):
        if resultvectors[i][j] == 1:
            PSFile.write("phaseshifter_in["+str(j)+"]")
            added = added + 1;
            if added < xorbits :
                PSFile.write(" ^ ")
            if added == xorbits :
                PSFile.write(";\n")
PSFile.write("\n")
PSFile.write("\n")
PSFile.write("endmodule //phaseshifter\n")


PSFile.close()