//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  bisttop.v
//  Revision        :  $Revision: 1.0 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <bisttop>
//  Description     :  <top module of bist>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************

module bisttop(/*AUTOARG*/
  // Outputs
  bist_en, bist_done, bist_scan_en, bist_scan_inputs, bisttop_scan_conf_out,
  // Inputs
  bist_clk, bist_rst_n, bist_start, bist_scan_outputs, bisttop_scan_conf_in,
  bisttop_scan_conf_en
  );

   `include "chip_params.v"
   
   //outputs
   output                       bist_en;
   output                       bist_done;
   output                       bist_scan_en;
   output [NUM_SCAN_CHAINS-1:0] bist_scan_inputs;
   output 			bisttop_scan_conf_out;
   
      
   //inputs
   input 			bist_clk;
   input 			bist_rst_n;
   input 			bist_start;
   input [NUM_SCAN_CHAINS-1:0] 	bist_scan_outputs;
   input 			bisttop_scan_conf_in;
   input 			bisttop_scan_conf_en;
   
   
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire                 bist_delaytest;         // From U_BISTCONTROLLER of bistcontroller.v
   wire [NUM_SCAN_CHAINS-1:0] bist_flipping_sel;// From U_BISTCONTROLLER of bistcontroller.v
   wire                 bist_lfsr_en;           // From U_BISTCONTROLLER of bistcontroller.v
   wire                 bist_misr_en;           // From U_BISTCONTROLLER of bistcontroller.v
   wire                 bist_scan_conf_out;     // From U_BISTCONTROLLER of bistcontroller.v
   wire [WEIGHTING_SEL-1:0] bist_weighting_sel; // From U_BISTCONTROLLER of bistcontroller.v
   wire [9:0]           concentrator_out;       // From U_CONSENTRATOR of concentrator.v
   wire [15:0]          flipping_out;           // From U_FLIPPING of flipping.v
   wire [9:0]           lfsr_out;               // From U_LFSR of lfsr.v
   wire                 lfsr_so;                // From U_LFSR of lfsr.v
   wire                 misr_so;                // From U_MISR of misr.v
   wire [15:0]          phaseshifter_out;       // From U_PHASESHIFTER of phaseshifter.v
   wire [15:0]          weighting_out;          // From U_WEIGHTING of weighting.v
   // End of automatics

   wire [LFSR_BITS-1:0] 		phaseshifter_in;
   wire [PS_OUTPUTS-1:0] 		weighting_in;
   wire [WEIGHTING_SEL-1:0] 		weighting_sel;
   wire [PS_OUTPUTS-1:0] 		flipping_in;
   wire [PS_OUTPUTS-1:0] 		flipping_sel;
   wire [PS_OUTPUTS-1:0] 		concentrator_in;
   wire [MISR_BITS-1:0] 		misr_in;

   wire                                 lfsr_en;
   wire                                 misr_en;
   wire                                 bist_scan_conf_en;
   wire                                 bist_scan_conf_in;
   wire                                 misr_clk;
   wire                                 lfsr_clk;
   wire                                 misr_rst_n;
   wire                                 lfsr_rst_n;
   wire                                 misr_load;
   wire                                 lfsr_load;
   wire                                 misr_si;
   wire                                 lfsr_si;

   /*AUTOREG*/

   //do the connection between the modules with assign statement, this is to
   //keep it more controllable when adding/removing modules in the bist logic
   //as AUTOINST would require modifiying the different modules.
   //this file could also be auto generated, but writing a script that does
   //this will take to much time, but it is possible...
   assign 		    phaseshifter_in = lfsr_out;
   assign 		    weighting_in = phaseshifter_out;
   assign 		    flipping_in = weighting_out;
   assign 		    bist_scan_inputs = flipping_out;

   assign 		    concentrator_in = bist_scan_outputs;
   assign 		    misr_in = concentrator_out;

   //connect bistcontroller to misr and lfsr
   assign 		    lfsr_en = bist_lfsr_en;
   assign 		    misr_en = bist_misr_en;
   assign 		    flipping_sel = bist_flipping_sel;
   assign 		    weighting_sel = bist_weighting_sel;

   assign 		    bist_scan_conf_en = bisttop_scan_conf_en;
   assign 		    bist_scan_conf_in = bisttop_scan_conf_in;
   
   
   //other misr and lfsr signals
   assign 		    misr_clk = bist_clk;
   assign 		    lfsr_clk = bist_clk;
   assign 		    misr_rst_n = bist_rst_n;
   assign 		    lfsr_rst_n = bist_rst_n;
   assign 		    misr_load = bisttop_scan_conf_en;
   assign 		    lfsr_load = bisttop_scan_conf_en;
   assign 		    misr_si = lfsr_so;
   assign 		    lfsr_si = bist_scan_conf_out;

   assign 		    bisttop_scan_conf_out = misr_so;
   
	  
   bistcontroller U_BISTCONTROLLER(/*AUTOINST*/
                                   // Outputs
                                   .bist_done           (bist_done),
                                   .bist_en             (bist_en),
                                   .bist_lfsr_en        (bist_lfsr_en),
                                   .bist_misr_en        (bist_misr_en),
                                   .bist_scan_en        (bist_scan_en),
                                   .bist_weighting_sel  (bist_weighting_sel[WEIGHTING_SEL-1:0]),
                                   .bist_flipping_sel   (bist_flipping_sel[NUM_SCAN_CHAINS-1:0]),
                                   .bist_delaytest      (bist_delaytest),
                                   .bist_scan_conf_out  (bist_scan_conf_out),
                                   // Inputs
                                   .bist_start          (bist_start),
                                   .bist_clk            (bist_clk),
                                   .bist_rst_n          (bist_rst_n),
                                   .bist_scan_conf_in   (bist_scan_conf_in),
                                   .bist_scan_conf_en   (bist_scan_conf_en));

   lfsr U_LFSR(/*AUTOINST*/
               // Outputs
               .lfsr_so                 (lfsr_so),
               .lfsr_out                (lfsr_out[9:0]),
               // Inputs
               .lfsr_clk                (lfsr_clk),
               .lfsr_rst_n              (lfsr_rst_n),
               .lfsr_en                 (lfsr_en),
               .lfsr_load               (lfsr_load),
               .lfsr_si                 (lfsr_si));

   
   phaseshifter U_PHASESHIFTER(/*AUTOINST*/
                               // Outputs
                               .phaseshifter_out(phaseshifter_out[15:0]),
                               // Inputs
                               .phaseshifter_in (phaseshifter_in[9:0]));



  
   flipping U_FLIPPING(/*AUTOINST*/
                       // Outputs
                       .flipping_out    (flipping_out[15:0]),
                       // Inputs
                       .flipping_in     (flipping_in[15:0]),
                       .flipping_sel    (flipping_sel[15:0]));
   
   weighting U_WEIGHTING(/*AUTOINST*/
                         // Outputs
                         .weighting_out         (weighting_out[15:0]),
                         // Inputs
                         .weighting_in          (weighting_in[15:0]),
                         .weighting_sel         (weighting_sel[1:0]));

   concentrator U_CONSENTRATOR(/*AUTOINST*/
                               // Outputs
                               .concentrator_out(concentrator_out[9:0]),
                               // Inputs
                               .concentrator_in (concentrator_in[15:0]));

   misr U_MISR(/*AUTOINST*/
               // Outputs
               .misr_so                 (misr_so),
               // Inputs
               .misr_clk                (misr_clk),
               .misr_rst_n              (misr_rst_n),
               .misr_en                 (misr_en),
               .misr_load               (misr_load),
               .misr_si                 (misr_si),
               .misr_in                 (misr_in[9:0]));

endmodule // bisttop

/*
 Local Variables:
 verilog-library-directories:(
 "."
 "../include"
 "../../../bistcontroller/custom/verilog/"
 "../../../lfsr/custom/verilog/"
 "../../../ps/custom/verilog/"
 "../../../weighting/custom/verilog/"
 "../../../flipping/custom/verilog/"
 "../../../concentrator/custom/verilog/"
 "../../../misr/custom/verilog/")
 End:
*/
