//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  clockmux2.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <clockmux2>
//  Description     :  <glitch free clock mux with select latches>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************


module clockmux2(/*AUTOARG*/
   // Outputs
   clk_out, 
   // Inputs
   clk1, clk2, clk_sel
   );

   //outputs
   output clk_out;

   //input
   input  clk1;
   input  clk2;
   input  clk_sel;

   reg 	  en1;
   reg 	  en2;

   always @ (/*AUTOSENSE*/clk1 or clk2 or clk_sel)
     begin
	if (clk1 == 1'b0 & clk_sel == 1'b1 & en2 == 1'b0)
	  en1 = 1'b1;
	else
	  if (clk1 == 1'b0 & clk_sel == 1'b0)
	    en1 = 1'b0;
	
	if (clk2 == 1'b0 & clk_sel == 1'b0 & en1 == 1'b0)
	  en2 = 1'b1;
	else
	  if (clk2 == 1'b0 & clk_sel == 1'b1)
	    en2 = 1'b0;
     end

   assign clk_out = (en1 & clk1) | (en2 & clk2);
   
endmodule // clockmux

   