//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  spi_co_processor_params.v
//  Revision        :  $Revision: 1.0 $
//  Checkin Date    :  $Date: 2008/05/28 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <>
//  Description     :  <parameter file for the CUT>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************

//values marked with FIXED require more change to code (FSM, counters...)
//DO NOT CHANGE THESE VALUES
//CPUBITS, DATA_PACKAGES and DATAPBIST ARE THE ONLY PARAMETERs TO BE CHANGED
//general parameters, bits and so on
parameter CPUBITS         = 64; // data width
parameter INSTRBITS       = 16; // FIXED - do not change
parameter OPCODEBITS      = 7;  // 7 of 16 bits   - FIXED - do not change
parameter ADDRBITS        = 3;  // 3*3 of 16 bits - FIXED - do not change
parameter SPIBITS         = 8;  // FIXED - do not change
parameter SPIBITCOUNT     = 3;  // FIXED - do not change
parameter DATA_PACKAGES   = CPUBITS/SPIBITS; //number of packages
parameter DATAPBITS       = 3;  // log2(DATA_PACKAGES)
parameter REGFILESIZE     = 8;  // number of registers - FIXED - do not change
parameter REGFILEADDRBITS = 3;  // log2(REGFILESIZE)   - FIXED - do not change


//ALU STATUS REGISTER PARAMETERS
parameter STATUSREGBITS   = 8;  // bits in status register
parameter SREG_C          = 7;  // carry bit
parameter SREG_Z          = 6;  // zero 
parameter SREG_N          = 5;  // negative
parameter SREG_V          = 4;  // 2's complement overflow
parameter SREG_S          = 3;  // N xor V
parameter SREG_H          = 2;  // not used
parameter SREG_b          = 1;  // not used
parameter SREG_c          = 0;  // not used

//CPU instructions ; THE TWO LAST BITS ARE DONT CARE when sent to the CPU, 
//but MUST be zero here, due to masking in instructionformatdecoder.v
parameter NOP       = 7'b0000000;
parameter WRITE     = 7'b0000100;
parameter READ      = 7'b0001000;
parameter WRITESREG = 7'b0001100;
parameter READSREG  = 7'b0010000;
parameter MOV       = 7'b0010100;
parameter ADD       = 7'b0100000;
parameter ADC       = 7'b0100100;
parameter SUB       = 7'b0101000;
parameter SBC       = 7'b0101100;
parameter MUL       = 7'b0110000;
parameter DIV       = 7'b0110100;
parameter MAC       = 7'b0111000;
parameter LSL       = 7'b1000100;
parameter LSR       = 7'b1001000;
parameter ROL       = 7'b1001100;
parameter ROR       = 7'b1010000;
parameter AND       = 7'b1010100;
parameter OR        = 7'b1011000;
parameter XOR       = 7'b1011100;
parameter INV       = 7'b1100000;

//register addresses
parameter R0        = 3'b000;
parameter R1        = 3'b001;
parameter R2        = 3'b010;
parameter R3        = 3'b011;
parameter R4        = 3'b100;
parameter R5        = 3'b101;
parameter R6        = 3'b110;
parameter R7        = 3'b111;
parameter RX        = 3'b000;


