//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  instructionformatdecoder.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <instructionformatdecoder>
//  Description     :  <split the instruction and addresses into 
//                     spearate signals>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************

module instructionformatdecoder(/*AUTOARG*/
   // Outputs
   alu_opcode, regfile_addr_a, regfile_addr_b, regfile_addr_d, 
   // Inputs
   instructionreg
   );

   `include "spi_co_processor_params.v"

   //outputs
   output [OPCODEBITS-1:0] alu_opcode;
   output [ADDRBITS-1:0]   regfile_addr_a;
   output [ADDRBITS-1:0]   regfile_addr_b;
   output [ADDRBITS-1:0]   regfile_addr_d;

   //inputs
   input [INSTRBITS-1:0]   instructionreg;
   
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   // End of automatics

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   // End of automatics

   //removing masking on alu opcode will result in low FC in delay test
   assign 		alu_opcode     = (instructionreg[INSTRBITS-1:INSTRBITS-OPCODEBITS] & {{(OPCODEBITS-2){1'b1}},2'b00});

  
   assign 		regfile_addr_d = instructionreg[3*ADDRBITS-1:2*ADDRBITS];
   assign 		regfile_addr_a = instructionreg[2*ADDRBITS-1:ADDRBITS];
   assign 		regfile_addr_b = instructionreg[ADDRBITS-1:0];


endmodule // instructionformatdecoder
