//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  registerfile.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <registerfile>
//  Description     :  <registerfile for the alu>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************

module registerfile(/*AUTOARG*/
   // Outputs
   regfile_alu_a, regfile_alu_b, regfile_alu_d, 
   // Inputs
   clk, rst_n, alu_regfile_write_en, regfile_addr_a, regfile_addr_b, 
   regfile_addr_d, alu_regfile_data
   );

   `include "spi_co_processor_params.v"

   //outputs
   output [CPUBITS-1:0]        regfile_alu_a;
   output [CPUBITS-1:0]        regfile_alu_b;
   output [CPUBITS-1:0]        regfile_alu_d;
   
   //inputs
   input 		       clk;
   input 		       rst_n;
   input 		       alu_regfile_write_en;
   input [REGFILEADDRBITS-1:0] regfile_addr_a; //address for output A
   input [REGFILEADDRBITS-1:0] regfile_addr_b; //address for output B
   input [REGFILEADDRBITS-1:0] regfile_addr_d; //address for input data
   input [CPUBITS-1:0] 	       alu_regfile_data;
 	       
   
   
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   // End of automatics

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   // End of automatics

   reg [CPUBITS-1:0] 	       regfile[REGFILESIZE-1:0];

   integer 		       i;
   
   always @ (posedge clk or negedge rst_n)
     begin
	if (rst_n == 1'b0)
	  begin
	     for (i = 0; i < REGFILESIZE; i = i + 1)
	       regfile[i] <= {{CPUBITS}{1'b0}};
	  end
	else
	  if (alu_regfile_write_en == 1'b1)
	    regfile[regfile_addr_d] <= alu_regfile_data;
     end // always @ (posedge clk)

   assign regfile_alu_a = regfile[regfile_addr_a];
   assign regfile_alu_b = regfile[regfile_addr_b];
   assign regfile_alu_d = regfile[regfile_addr_d];
   
   
endmodule // registerfile
