//****************************************************************************
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the agreement which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than the one
//           mentioned in the agreement, as the term is defined in the
//        agreement, is strictly forbidden without a prior written consent.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  spislave.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project         :  Master Thesis
//  Module name     :  <spislave>
//  Description     :  <the spislave>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************

module spislave(/*AUTOARG*/
   // Outputs
   sdo, spi_data, spi_fsm_load, ss_n_sync, 
   // Inputs
   sclk, clk, spi_rst_n, sdi, ss_n, data_spi, fsm_spi_load
   );

   `include "spi_co_processor_params.v"
   
   //outputs
   output               sdo;
   output [SPIBITS-1:0] spi_data;
   output 		spi_fsm_load;
   output 		ss_n_sync;
		
   //inputs
   input 		sclk;
   input 		clk;
   input 		spi_rst_n;
   input 		sdi;
   input 		ss_n;
   input [SPIBITS-1:0]	data_spi;
   input 		fsm_spi_load;
   
	

   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   // End of automatics

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   reg			spi_fsm_load;
   // End of automatics
 	
   //other regs and wires
   reg [2:0] 		sclk_buf;
   wire 		sclkposedge;
   reg [1:0] 		sdi_buf ;
   wire 		sdi_sync;
   reg [1:0] 		ss_n_buf;
   wire 		ss_n_sync;
   
   reg [SPIBITS-1:0] 	 shiftreg;
   reg [SPIBITCOUNT-1:0] shiftcount;
   reg 			 shiftcountlast;

   

   /*
    * sclk sampling
    * 
    */
   always @ (posedge clk or negedge spi_rst_n)
     begin
	if (spi_rst_n == 1'b0)
	  begin
	     sclk_buf <= 3'b000;
	  end
	else
	  begin
	     sclk_buf <= {sclk_buf[1:0],sclk};
	  end
     end // always @ (posedge clk)

   assign sclkposedge = (sclk_buf[2:1] == 2'b01);

   /*
    * sdi sampling
    * 
    */
   always @ (posedge clk or negedge spi_rst_n)
     begin
	if (spi_rst_n == 1'b0)
	  sdi_buf <= 2'b00;
	else
	  sdi_buf <= {sdi_buf[0],sdi};
     end
   assign sdi_sync = sdi_buf[1];
   
   /*
    * ss_n sampling
    * 
    */
   always @ (posedge clk or negedge spi_rst_n)
     begin
	if (spi_rst_n == 1'b0)
	  ss_n_buf <= 2'b00;
	else
	  ss_n_buf <= {ss_n_buf[0],ss_n};
     end
   assign ss_n_sync = ss_n_buf[1];
   
   /*
    * 8 bit input shift register, sdi and sdo
    * 
    */
   always @ (posedge clk or negedge spi_rst_n)
     begin
	if (spi_rst_n == 1'b0)
	  begin
	     shiftreg <= {{SPIBITS}{1'b0}};
	  end
	else
	  begin
	     if (fsm_spi_load == 1'b1)
	       shiftreg <= data_spi;
	     else
	       if ((sclkposedge == 1'b1) && (ss_n_sync == 1'b0))
		 begin
		    shiftreg <= {shiftreg[SPIBITS-2:0],sdi_sync};
		 end
	  end // else: !if(spi_rst_n == 1'b0)
     end // always @ (posedge clk)

   assign sdo = (ss_n_sync) ? 1'bz: shiftreg[SPIBITS-1]; //tristate output
   assign spi_data = shiftreg;

   /*
    * shift counter and output ready signal
    * 
    */
   always @ (posedge clk or negedge spi_rst_n)
     begin
	if (spi_rst_n == 1'b0)
	  begin 
	     shiftcount <= {{SPIBITCOUNT}{1'b1}};
	     shiftcountlast <= 1'b1;
	  end
	else
	  begin
	     if (ss_n_sync == 1'b1)
	       begin
		  shiftcount <= {{SPIBITCOUNT}{1'b1}};
		  shiftcountlast <= 1'b1;
	       end
	     else
	       if ((sclkposedge == 1'b1) && (ss_n_sync == 1'b0))
		 shiftcount <= shiftcount + 3'b001;
	     shiftcountlast <= shiftcount[0]&shiftcount[1]&shiftcount[2];
	  end
     end
  
   always @ (/*AUTOSENSE*/shiftcount or shiftcountlast)
     begin
	if (shiftcount == 3'b111 && shiftcountlast == 1'b0)
	  spi_fsm_load = 1'b1;
	else
	  spi_fsm_load = 1'b0;
     end
      
endmodule // spislave
