set higheffort 1
set gate_clock 1
set insert_scan 1
set corner "max"
set numScanChains 16

sh date

set top_module spi_co_processor

if {$corner == "max"} {
  set sclib LLSClib-max+ind
} else {
  set sclib LLSClib-max+ind
}

#Save info needed by rtldrc
set hdlin_enable_rtldrc_info true

#
#Formality svf file
#
#set_svf ../../ntl/${top_module}_hier.svf


set synthetic_library dw_foundation.sldb


lappend search_path ./

#set link_library [concat * [glob ./*.asdf]]


lappend synthetic_library dw_foundation.sldb
#lappend link_library dw_foundation.sldb


set target_library /home/teleistad/local_libs/synt/${sclib}.db
set symbol_library /home/teleistad/local_libs/synt/${sclib}.sdb


set glob_files [glob ../../rtl/module/dut/custom/verilog/spi_co_processor.v \
                     ../../rtl/module/dut/custom/verilog/spi_co_proc_if.v   \
                     ../../rtl/module/dut/custom/verilog/spi_co_proc_fsm.v  \
                     ../../rtl/module/dut/custom/verilog/alu.v              \
                     ../../rtl/module/dut/custom/verilog/instructionformatdecoder.v \
                     ../../rtl/module/dut/custom/verilog/spislave.v         \
                     ../../rtl/module/dut/custom/verilog/registerfile.v]
#../../rtl/module/dut/custom/verilog/fifo.v
foreach i $glob_files {
    if ![regexp (CVS) $i] {
        lappend verilog_files $i
    }
}


#Suppress warnings
#
#Warning:  Case branches are unreachable. , lots of these in cpu_funcs
suppress_message ELAB-314
#Warning:  Starting with the 2000.11-1 release, the Presto Verilog reader treats 
# Verilog 'integer' types as signed, Ancient history
suppress_message VER-314
#Warning:  signed to unsigned conversion occurs., Used all over for reset and state machines.
suppress_message VER-318
#Statement unreacheable
suppress_message VER-61
#Constant push through sequential cell.
suppress_message OPT-504
#Value of function may be used before being assigned; the synthesized result may not match simulations.
suppress_message ELAB-393
#Variable may be read before being assigned; the synthesized result may not match simulations. 
suppress_message ELAB-293


#
# Then read in files
#
set include_path "../../rtl/module/dut/custom/include/"
foreach i $glob_files {
    lappend include_path $i
}
set search_path ". $include_path $search_path"


# Use module compiler to generate DesignWares, if licensed
set dw_prefer_mc_inside true

# Clock gating style
set_clock_gating_style -sequential_cell latch -min 3 -control_point before -control_signal scan_enable


define_design_lib WORK -path work
set result [analyze -format verilog $verilog_files ]
if [expr $result == 0] quit



#
# Elaborate
#

set result [elaborate $top_module]

set result [link]
if [expr $result == 0] quit

if { [is_true $gate_clock ] } {
    insert_clock_gating
    current_design $top_module
}

set_auto_disable_drc_nets -clock true -constant true -scan true

check_design

write -xg_force_db -hierarchy -output ./${top_module}_elab.db


set_dont_use  [format "%s%s"  $sclib /*da]

uniquify

if { [is_true $insert_scan ] } {
  hookup_testports -verbose
}

current_design $top_module

#
# Set constrants and clocks
# 
source  -echo -verbose ./dut_constraints.tcl


#
# insert scan chain
#
if { [is_true $insert_scan] } {

    set_scan_configuration -style multiplexed_flip_flop -chain_count $numScanChains
    #set_scan_configuration -external_tri disable_all
    set_dft_signal -view existing_dft -type ScanClock -port clk -timing [list 45 55]
    set_dft_signal -view existing_dft -type Reset -port rst_n -active_state 0
    create_test_protocol > reports/scan_pre_comp_create_test_protocol.log
    dft_drc -verbose > reports/scan_pre_comp_dft_drc.log
}

#
# Compile
#

if { [is_true $higheffort ] } {
    set_critical_range 0.2 ${top_module}
    set_resource_implementation use_fastest
    set_ultra_optimization true
    set_max_area 2000
    
    if {[is_true $insert_scan ]} {
        compile_ultra -scan -no_autoungroup
        #compile -scan -map_effort high
    } else {
        compile_ultra -no_autoungroup
        #compile -map_effort high
    }
} else {
    if {[is_true $insert_scan]} {
        compile -scan 
    } else {
        compile 
    }
}

if { [is_true $insert_scan] } {
    create_test_protocol > reports/scan_post_comp_create_test_protocol.log
    dft_drc -verbose > reports/scan_post_comp_dft_drc.log
    
    #for {set i 1} {$i < 10} {incr i}
    #{
    set_dft_signal -view spec -type ScanDataIn -port TEST_SI
    #${i}
    set_dft_signal -view spec -type ScanDataOut -port TEST_SO
    #${i}
    set_dft_signal -view spec -type ScanEnable -port TEST_SE
    #${i}
    #}
    preview_dft > reports/scan_preview_dft.log
    insert_dft > reports/scan_insert_dft.log
    dft_drc -verbose > reports/scan_last_dft_drc.log
    
    estimate_test_coverage > reports/scan_test_coverage.log

}



#
# Save DB
#
write -xg_force_db -hierarchy -output ./${top_module}_hier.db
write_sdc ${top_module}.sdc

#
# Save some more stuff
#
write -format verilog -hierarchy -output ${top_module}.v
write_test_protocol -output ${top_module}.spf
write_sdf ${top_module}.sdf

#Report
report_area > reports/plain_area.log
report_area -hierarchy > reports/plain_area_hierarchy.log
report_clocks > reports/plain_clocks.log
report_timing > reports/plain_timing.log

if { [is_true $insert_scan] } {
	report_dft_configuration > reports/dft_configuration.log
	report_scan_path -view existing_dft -chain all > reports/scan_chains.log
	report_scan_path -view existing_dft -cell all > reports/scan_cells.log

}

quit

