/*
 * Validation program form SPI_CO_PROCESSOR
 *
 * Runs on AVR connected to FPGA through SPI
 *
 *
 */

#include "avr_spi_co_processor.h"

/*
 * globale variables
 */
struct spipack spipackage;
struct spipack *spi_pack_p = &spipackage;
uint32_t data = 0;

/*
 * USART/UART routines for debugging
 */
void init_uart(void)
{
	UBRRL = (F_CPU / (16 * UART_BAUD)) - 1;
	UCSRB = (1<<TXEN) | (1<<RXEN); /* tx/rx enable */
}

int uart_putchar(char c) //, FILE *stream
{
	if (c == '\n')
    	uart_putchar('\r'); //, stream
	UDR = c;
    loop_until_bit_is_set (UCSRA,UDRE);
    
    return 0;
}

int uart_string(char* str)
{
	while(*str)
		uart_putchar(*str++);
}

/*
 * SPI routines for communication with FPGA
 */
void init_spi()
{
	//set MOSI,SS and SCK as output, all others as inputs
	DDR_SPI = (1<<DD_MOSI) | (1<<DD_SCK) | (1<<DD_SS);
	//ENABLE SPI MASTER, SET SCK to FCK / 128
	SPCR = (1<<SPE) | (1<<MSTR) | (1<<SPR0) | (1<<SPR1);	

	return;
}

uint8_t spi_txrx(uint8_t txdata)
{
	//send data
	SPDR = txdata;
	//wait for send complete
	while(!(SPSR & (1<<SPIF)))
		;
	//received data
	return SPDR;
}

void send(uint8_t cmd, uint8_t rd, uint8_t ra, uint8_t rb, uint32_t data)
{
	if (rd & 0x04) //check if the msb of the 3 bit address rd is set
		spi_pack_p->instr[0] = (cmd<<1)|0x01;
	else
		spi_pack_p->instr[0] = (cmd<<1);

	spi_pack_p->instr[1] = (rd<<6) | (ra << 3) | (rb << 0);

	char strbuf[16];
	uart_string("instr: 0x");
	if(spi_pack_p->instr[0] < 16)
		uart_string("0");
	itoa(spi_pack_p->instr[0],strbuf,16);
	uart_string(strbuf);
	if(spi_pack_p->instr[1] < 16)
		uart_string("0");
	itoa(spi_pack_p->instr[1],strbuf,16);
	uart_string(strbuf);
	uart_string("\n");

	uint8_t i;
	for (i = 0; i < 4; i++)
    {
        spi_pack_p->data[i] = (data>>8*(3-i));
    }

    PORTB &= ~(1<<SSP); //set ss_n low
    spi_pack_p->instr[0] = spi_txrx(spi_pack_p->instr[0]);
    spi_pack_p->instr[1] = spi_txrx(spi_pack_p->instr[1]);
    spi_pack_p->data[0] = spi_txrx(spi_pack_p->data[0]);
    spi_pack_p->data[1] = spi_txrx(spi_pack_p->data[1]);
    spi_pack_p->data[2] = spi_txrx(spi_pack_p->data[2]);
    spi_pack_p->data[3] = spi_txrx(spi_pack_p->data[3]);
    PORTB |= (1<<SSP); // set ss_n high
    
	return;
}

void spi_dtoi()
{
 	data = 	(spi_pack_p->data[0]<<24) | \
			(spi_pack_p->data[1]<<16) | \
			(spi_pack_p->data[2]<< 8) | \
			(spi_pack_p->data[3]<< 0);
	return;
}


/*
 * CRASH routing, stops the program and display error on LEDs
 */
void dispError(uint8_t error)
{
	PORTC = 0xFF;
	PORTC = ~error;
	
	while(1)
	{
		PORTC = 0xFF;
		//_delay_ms(100);
		PORTC = ~error;
	}
	
}

/*
 * MAIN 
 */
int main(void)
{
	//init hardware
	init_spi();
	init_uart();
	//set port D as output for LED debugging
	DDRC = 0xFF;

	uart_string("\n\nStarting test program\n");

	//variable used to convert int/long to strings
	char strbuf[16];

	//variables used as data
	uint32_t var1 = 1;
	uint32_t var2 = 2;

	uint32_t foo32 = 0;
	uint8_t  foo8  = 0;
	uint32_t bar = 0;

	//start test routine

	//write/read test
	for (foo8 = 0; foo8 < 8; foo8++)
	{
		send(WRITE, foo8, RX,RX, foo32);

		uart_string("wrote ");
		ltoa(foo32,strbuf,10);
		uart_string(strbuf);
		uart_string(" to address ");
		ltoa(foo8,strbuf,10);
		uart_string(strbuf);
		uart_string("\n");

		foo32++;
	}
	foo32 = 0;

	for (foo8 = 0; foo8 < 8; foo8++)
	{
		send(READ,RX,foo8,RX,0);
		spi_dtoi();
		bar = data;

		uart_string("read ");
		ltoa(bar,strbuf,10);
		uart_string(strbuf);
		uart_string(" from address ");
		ltoa(foo8,strbuf,10);
		uart_string(strbuf);
		uart_string("\n");		

		_delay_ms(100);
		if (!(foo32 == bar))
		{
			ltoa(foo32,strbuf,10);
			uart_string(strbuf);
			uart_string(" ");
			ltoa(bar,strbuf,10);
			uart_string(strbuf);
			dispError(1);
		}
		foo32++;
	}
	
	//ADD
	send(ADD,R1,R1,R1,0); //R1 = 1, 1+1= 2
	send(READ,RX,R1,RX,0);
	spi_dtoi();
	bar = data;
	if(!(bar == 2))
		dispError(2);

	//SUB
	send(SUB,R2,R2,R2,0); //R2 = 2, 2-2= 0
	send(READ,RX,R2,RX,0);
	spi_dtoi();
	bar = data;
	if(!(bar == 0))
		dispError(3);
	
	//MUL
	send(MUL,R3,R3,R3,0); //R3 = 3, 3*3= 9
	send(READ,RX,R3,RX,0);
	spi_dtoi();
	bar = data;
	if(!(bar == 9))
		dispError(4);
	
	//MAC
	send(MAC,R4,R4,R4,0); //R4 = 4, 4*4 + 4= 20
	send(READ,RX,R4,RX,0);
	spi_dtoi();
	bar = data;
	if(!(bar == 20))
		dispError(5);

	//LSL
	send(LSL,R5,R5,RX,0); //R5 = 5, lsl -> 10
	send(READ,RX,R5,RX,0); 
	spi_dtoi();
	bar = data;
	if(!(bar == 10))
		dispError(6);

	//LSR
	send(LSR,R5,R5,RX,0); //R5 = 10, lsr -> 5
	send(READ,RX,R5,RX,0); 
	spi_dtoi();
	bar = data;
	if(!(bar == 5))
		dispError(7);
	
	
	

	while(1)
	{
		//display some counting on the STK500 LEDs

    	//cmd, Rd, Ra, Rb, {data} 
        send(WRITE, R0,RX,RX, var1);
        send(WRITE, R1,RX,RX, var2);
		send(ADD,R2,R1,R0,0);
        send(READ,RX,R2,RX,0);
  
		PORTC = 0xFF; //clear LEDS
		PORTC = ~spi_pack_p->data[3]; //set new LED output
		_delay_ms(1); //delay for some time
   
   		//update var1 with the received value
		spi_dtoi();
		var1 = data;
	}	
		
}
