//****************************************************************************
//                 PROPRIETARY INFORMATION OF ATMEL CORPORATION
//
//                     
//                       Copyright Atmel Corporation
//
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the LICENSE which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than to the
//           EMPLOYEES of the Licensee, as the term is defined in the
//        LICENSE, is strictly forbidden without a prior written consent
//                            of Atmel Corporation.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  testbench.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project number  :  ? , Master Thesis
//  Module name     :  <testbench>
//  Description     :  <test the dut, spi_co_processor>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************


module testbench();

   `include "bistcontroller_params.v"
   `include "chip_params.v"
   
   parameter clock_period = 2;
   
   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   // End of automatics

   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire			bist_done;		// From U_BISTTOP of bisttop.v
   wire			bist_en;		// From U_BISTTOP of bisttop.v
   wire			bist_scan_en;		// From U_BISTTOP of bisttop.v
   wire			bisttop_scan_conf_out;	// From U_BISTTOP of bisttop.v
   wire [NUM_SCAN_CHAINS-1:0]scan_inputs;	// From U_BISTTOP of bisttop.v
   wire [NUM_SCAN_CHAINS-1:0]scan_outputs;	// From U_TESTDUT of testdut.v
   // End of automatics

   //regs for inputs
   reg 			   bist_start;
   reg 			   bisttop_scan_conf_in;
   reg 			   bisttop_scan_conf_en;
   wire 		   bist_clk;
   wire 		   bist_rst_n;
   
   //wires for modul inputs

   //other regs
   reg 			clk;
   reg 			rst_n;

   assign 		bist_clk = clk;
   assign 		bist_rst_n = rst_n;
   
   
   /*
    * Generate clock
    *
    */
   initial begin
      clk = 'b1;
   end
   always #(clock_period/2) clk = !clk;
   
   /*
    * Generate reset
    *
    */
   initial begin
      rst_n = 'b0;
      #100;
      rst_n = 'b1;
   end
     
   /*
    * VCD logging
    *
    */
   initial begin
      $display("Enable logging.");
      #1; //this tick contains just useless stuff...
      $vcdpluson(0); //use with design compiler
      //$dumpfile("vsim.vcd"); 
      //$dumpvars(5,U_SPI_CO_PROCESSOR);
      $dumpvars;
      
   end

   /*
    * Simulation time out
    *
    */
   initial begin
      #20000;
      $finish;
   end

   /*
    * TEST sequencs
    *
    */
   initial begin
      bisttop_scan_conf_en <= 1'b0;
      bisttop_scan_conf_in <= 1'b0;
      bist_start <= 1'b0;
      //wait for reset
      #200;
    
      bist_start <= 1'b1;
      #(clock_period);
      bist_start <= 1'b0;

      #4000;
      
      bist_start <= 1'b1;
      #(clock_period);
      bist_start <= 1'b0;

      #4000;

      bist_start <= 1'b1;
      #(clock_period);
      bist_start <= 1'b0;

      
      
      //$display("LFSR output:");
      //$monitor("%t: [%x]",$time,`TB.lfsr_out);
   end // initial begin
   
   //DUT/CUT moodul
   bisttop U_BISTTOP(/*AUTOINST*/
		     // Outputs
		     .bist_en		(bist_en),
		     .bist_done		(bist_done),
		     .bist_scan_en	(bist_scan_en),
		     .bist_scan_inputs	(scan_inputs[NUM_SCAN_CHAINS-1:0]),
		     .bisttop_scan_conf_out(bisttop_scan_conf_out),
		     // Inputs
		     .bist_clk		(bist_clk),
		     .bist_rst_n	(bist_rst_n),
		     .bist_start	(bist_start),
		     .bist_scan_outputs	(scan_outputs[NUM_SCAN_CHAINS-1:0]),
		     .bisttop_scan_conf_in(bisttop_scan_conf_in),
		     .bisttop_scan_conf_en(bisttop_scan_conf_en));
   
   testdut U_TESTDUT(/*AUTOINST*/
		     // Outputs
		     .scan_outputs	(scan_outputs[NUM_SCAN_CHAINS-1:0]),
		     // Inputs
		     .bist_clk		(bist_clk),
		     .bist_rst_n	(bist_rst_n),
		     .bist_scan_en	(bist_scan_en),
		     .scan_inputs	(scan_inputs[NUM_SCAN_CHAINS-1:0]));
   
endmodule // testbench

/*
 Local Variables:
 verilog-library-directories:(
 "."
 "../../rtl/module/bisttop/custom/verilog/")
 End:
*/
