//****************************************************************************
//                 PROPRIETARY INFORMATION OF ATMEL CORPORATION
//
//                     
//                       Copyright Atmel Corporation
//
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the LICENSE which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than to the
//           EMPLOYEES of the Licensee, as the term is defined in the
//        LICENSE, is strictly forbidden without a prior written consent
//                            of Atmel Corporation.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  testbench.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project number  :  ? , Master Thesis
//  Module name     :  <testbench>
//  Description     :  <test the dut, spi_co_processor>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************


module testbench();

`include "chip_params.v"
`include "spi_co_processor_params.v"
`include "bistcontroller_params.v"

  integer 		i;
  
  parameter clock_period = 20;
  parameter spiclock_period = clock_period * 100;

  //enable/disable VCD output
  parameter VCD_OUTPUT = 0;

  //PI isolation
  //`define PIiso 1
  
  //file IO
  integer    fdscn;
  integer    fdseq;
  integer    fdpis;
  integer    fdpos;

  //counter used to count patterns written to files
  integer    patterncounter = 0;

  //variable used to determine if bist is started 
  //used to prevent writing incorrect pattern at time #0
  integer    bist_started = 0;
  //number of patterns generated by bist logic, will be dependent on the bist conf_r register
  //if all test are run, m=(number_of_weights-1)*2+1
  //n it the number of patterns to run for each test
  //numpat = (PATTERNS+1)*((WEIGHTING_WEIGHTS-1)*2+1)-1;
  //integer    numpat = (PATTERNS+1)*((WEIGHTING_WEIGHTS-1)*2+1)-1;
  integer numpat = 6138;
  
  //pattern types, uses fmodel (faultmodel) as argument when writing to files 
  parameter  STUCK = 0,
             TRAN  = 1,
             PATH  = 2;
  parameter  fmodel = TRAN;
  
  /*AUTOREG*/

  /*AUTOWIRE*/
  // Beginning of automatic wires (for undeclared instantiated-module outputs)
  wire                  bist_done;              // From U_CHIP of chip.v
  wire                  conf_scan_out;          // From U_CHIP of chip.v
  wire [2:0]            debug;                  // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire [NUM_EXT_SCAN_CHAINS-1:0] ext_scan_out;  // From U_CHIP of chip.v
  wire                  osc_clk;                // From U_OSC of osc.v
  wire                  scan_en;                // From U_CHIP of chip.v
  wire [NUM_SCAN_CHAINS-1:0] scan_inputs;       // From U_CHIP of chip.v
  wire                  sdo;                    // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  sys_clk;                // From U_CHIP of chip.v
  wire                  test_so1;               // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so10;              // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so11;              // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so12;              // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so13;              // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so15;              // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so16;              // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so2;               // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so3;               // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so4;               // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so5;               // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so6;               // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so7;               // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so8;               // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  wire                  test_so9;               // From U_SPI_CO_PROCESSOR of spi_co_processor.v
  // End of automatics

  wire                  test_si1;
  wire                  test_si2;
  wire                  test_si3;
  wire                  test_si4;
  wire                  test_si5;
  wire                  test_si6;
  wire                  test_si7;
  wire                  test_si8;
  wire                  test_si9;
  wire                  test_si10;
  wire                  test_si11;
  wire                  test_si12;
  wire                  test_si13;
  wire                  test_si14;
  wire                  test_si15;
  wire                  test_si16;

  //wire                  test_so15;
  //assign                test_so15 = debug[2];  
  wire                  test_so14;
  assign                test_so14 = debug[2];
  
  
  
  reg [LEN_SCAN_CHAINS-1:0] chain_in[NUM_SCAN_CHAINS-1:0];
  reg [SHIFT_BITS-1:0] chain_out[NUM_SCAN_CHAINS-1:0];
  wire [6+NUM_SCAN_CHAINS-1:0] pis;
  wire [4+NUM_SCAN_CHAINS-1-1:0] pos;
  
  wire  test_se;
  wire  [NUM_SCAN_CHAINS-1:0] scan_outputs;

  assign test_se = scan_en;
  
  assign scan_outputs = {test_so16,
                         test_so15,
                         test_so14,
                         test_so13,
                         test_so12,
                         test_so11,
                         test_so10,
                         test_so9,
                         test_so8,
                         test_so7,
                         test_so6,
                         test_so5,
                         test_so4,
                         test_so3,
                         test_so2,
                         test_so1};
   
   assign test_si1 = scan_inputs[0];
   assign test_si2 = scan_inputs[1];
   assign test_si3 = scan_inputs[2];
   assign test_si4 = scan_inputs[3];
   assign test_si5 = scan_inputs[4];
   assign test_si6 = scan_inputs[5];
   assign test_si7 = scan_inputs[6];
   assign test_si8 = scan_inputs[7];
   assign test_si9 = scan_inputs[8];
   assign test_si10 = scan_inputs[9];
   assign test_si11 = scan_inputs[10];
   assign test_si12 = scan_inputs[11];
   assign test_si13 = scan_inputs[12];
   assign test_si14 = scan_inputs[13];
   assign test_si15 = scan_inputs[14];
   assign test_si16 = scan_inputs[15];
  
   
   //regs for modul inputs
   //reg [:] name;
//`ifdef PIiso
  //wire    ss_n;
  //wire    sdi;
  //wire    sclk;
//`else
  reg     ss_n;
  reg     sdi;
  reg     sclk;
//`endif
      
  reg                   ext_test_en;
  reg                   ext_scan_en;
  reg [NUM_EXT_SCAN_CHAINS-1:0] ext_scan_in;
  reg                   bist_start;
     
  reg                   conf_scan_in;
  reg                   conf_scan_en;
   //wires for modul inputs

   //other regs
  reg 			ext_clk;
  reg 			rst_n;
  reg 			sclk_run;

  reg [INSTRBITS-1:0]   cmd;
  reg [CPUBITS-1:0]     data;
  reg [8+PATTERNCOUNTER_BITS+LEN_SCAN_CHAINS+1+CONFCOUNTER+NUM_SCAN_CHAINS+LFSR_BITS+MISR_BITS-1:0] confdata;
  
  
  wire                  clk;
  
  assign                clk = sys_clk;



    //PIS and POS assignment
  assign                pis = {rst_n,
                               clk,
                               sdi,
                               ss_n,
                               sclk,
                               test_se,
                               test_si1,
                               test_si2,
                               test_si3,
                               test_si4,
                               test_si5,
                               test_si6,
                               test_si7,
                               test_si8,
                               test_si9,
                               test_si10,
                               test_si11,
                               test_si12,
                               test_si13,
                               test_si14,
                               test_si15,
                               test_si16};
  
  assign                pos = {sdo,debug,
                               test_so1,
                               test_so2,
                               test_so3,
                               test_so4,
                               test_so5,
                               test_so6,
                               test_so7,
                               test_so8,
                               test_so9,
                               test_so10,
                               test_so11,
                               test_so12,
                               test_so13,
                               test_so15,
                               test_so16}; //test_so15 at debug[2],



   /*
    * Task for sending cmd
    * 
    */
   
   task sendcmd;
      input [6:0] instr;
      input [2:0] addrd;
      input [2:0] addra;
      input [2:0] addrb;
      begin
	 cmd <= {instr,addrd,addra,addrb};
	 ss_n <= 1'b0;
	 #(spiclock_period/2);
	 sclk_run <= 'b1;
	 
	 for(i = 0; i<=15; i=i+1)
	   begin
	      sdi <= cmd[15];
	      cmd <= {cmd[14:0],sdo};
	      @(negedge sclk);
	   end
	 sclk_run <= 'b0;
	 #(spiclock_period/2);
	 ss_n <= 'b1;
      end
   endtask // sendcmd


   /*
    * Task for sending cmd and data
    * also used to receive data
    */
   task sendcmddata; //(instr,addrd,addra,addrb,data);
      input [6:0] instr;
      input [2:0] addrd;
      input [2:0] addra;
      input [2:0] addrb;
      input [31:0] in;
      begin
	 cmd <= {instr,addrd,addra,addrb};
	 data <= in;
	 ss_n <= 1'b0;
	 #(spiclock_period/2);
	 sclk_run <= 'b1;
	 
	 for(i = 0; i<=15; i=i+1)
	   begin
	      sdi <= cmd[15];
	      cmd <= {cmd[14:0],sdo};
	      @(negedge sclk);
	   end
	 for(i = 0; i<=(CPUBITS-1); i=i+1)
	   begin
	      //sdi <= data[CPUBITS-1-i];
	      sdi <= data[CPUBITS-1];
	      data <= {data[CPUBITS-2:0],sdo};
	      @(negedge sclk);
	   end
	 sclk_run <= 'b0;
	 #(spiclock_period/2);
	 ss_n <= 'b1;
      end
   endtask // sendcmd




   /*
    * Task for scanning in new configuration to clock controller, bistcontroller,
    * lfsr and misr
    *
    */

   task confchange;
      input [7:0] clockconf;
      input [PATTERNCOUNTER_BITS-1:0] patterns_to_apply;
      input [LEN_SCAN_CHAINS:0]       bits_to_apply;
      input [CONFCOUNTER-1:0]         bist_conf;
      input [NUM_SCAN_CHAINS-1:0]     flipping_conf;
      input [LFSR_BITS-1:0]           lfsr_seed;
      input [MISR_BITS-1:0]           misr_seed;
     
     begin
         confdata <= {misr_seed,lfsr_seed,flipping_conf,bist_conf,bits_to_apply,patterns_to_apply,clockconf};
	 ext_test_en <= 1'b1;

	 @(negedge ext_clk);
	 @(posedge ext_clk);
	 @(negedge ext_clk);
	 @(posedge ext_clk);
	 	 
	 conf_scan_en <= 1'b1;

	 @(negedge ext_clk);
	 @(posedge ext_clk);
	 @(negedge ext_clk);
	 @(posedge ext_clk);
	 
	 for(i = 0; i<=(8+PATTERNCOUNTER_BITS+LEN_SCAN_CHAINS+1+CONFCOUNTER+NUM_SCAN_CHAINS+LFSR_BITS+MISR_BITS-1); i=i+1)
	   begin
	      conf_scan_in <= confdata[8+PATTERNCOUNTER_BITS+LEN_SCAN_CHAINS+1+CONFCOUNTER+NUM_SCAN_CHAINS+LFSR_BITS+MISR_BITS-1];
              confdata <= {confdata[8+PATTERNCOUNTER_BITS+LEN_SCAN_CHAINS+1+CONFCOUNTER+NUM_SCAN_CHAINS+LFSR_BITS+MISR_BITS-2:0],conf_scan_out};
	      @(posedge ext_clk);
	      
	   end
	 
	 conf_scan_en <= 1'b0;

	 @(negedge ext_clk);
	 @(posedge ext_clk);
	 @(negedge ext_clk);
	 @(posedge ext_clk);
	 
	 ext_test_en <= 1'b0;
	 
	 @(negedge ext_clk);
	 
      end
   endtask // sendcmd

  
  
   /*
    * Generate SPI clock
    *
    */
/* -----\/----- EXCLUDED -----\/-----
   initial begin
      sclk = 'b0;
      sclk_run = 'b0;
   end
   always #(spiclock_period/2) if(sclk_run == 'b1 ) sclk = !sclk;
 -----/\----- EXCLUDED -----/\----- */
   
   
 
   /*
    * Generate clock
    *
    */
   initial begin
      ext_clk = 'b1;
   end
   always #(clock_period/2) ext_clk = !ext_clk;
   
   /*
    * Generate reset
    *
    */
   initial begin
      rst_n <= 1'b0;
      #1000;
      rst_n <= 1'b1;
   end
     
   /*
    * VCD logging
    *
    */
   initial begin
     #1; //this tick contains just useless stuff...
     if(VCD_OUTPUT)
       begin
         $display("Enable logging."); 
         $vcdpluson(0);         //use with design compiler
                                //$dumpfile("vsim.vcd"); 
         //$dumpvars(5,U_SPI_CO_PROCESSOR);
         
         //$dumpvars;
         //$dumpports(U_SPI_CO_PROCESSOR, "vcde_output.evcd");
       end                     
   end

   /*
    * TEST sequencs
    *
    */
/* -----\/----- EXCLUDED -----\/-----
`ifdef PIiso
  assign sdi = test_si11;
  assign sclk = test_si12;
  assign ss_n = test_si13;
`endif
 -----/\----- EXCLUDED -----/\----- */
   
  initial begin
     //sett all PI
     //PI DUT
//`ifndef PIiso
     sdi = 1'b0;
     sclk = 1'b0;
     ss_n = 1'b0;
//`endif
     
     //PI BIST/CLOCK-CONTROLLER
     ext_test_en = 1'b0;
     ext_scan_en = 1'b0;
     ext_scan_in = {NUM_EXT_SCAN_CHAINS{1'b0}};
     bist_start = 1'b0;
     
     conf_scan_in = 1'b0;
     conf_scan_en = 1'b0;

     
     #1500; //wait for reset
     
     //scan in bist/clock config
     //clockconf, numpatterns, numbits, bistconf, flipping, lfsr seed, misr seed
     //confchange(8'hFF, 16'h03FF, 26'h000019, 8'h80, 16'hFFFF, 10'h3FF, 10'h000);
     //numpat = 1022;
     
     #1000;
     
     bist_start = 1'b1;
     #(1000);
     bist_start = 1'b0;
     
/* -----\/----- EXCLUDED -----\/-----
      ss_n <= 1'b1;
      sdi <= 1'b0;
      
      //wait for reset at #100
      #200;
      sendcmddata(WRITE,R0,RX,RX,32'h00000001);
      #1000;
      sendcmddata(WRITE,R1,RX,RX,32'h00000001);
      #1000;
      sendcmd(ADD,R2,R1,R0);
      #1000;
      sendcmddata(READ,RX,R2,RX,32'hAAAA5555);
      #1000;
 -----/\----- EXCLUDED -----/\----- */


     
      //$fmonitor(fd1, "time=%d : out=%4h, in=%6h", $time, scan_inputs, chain_in[0]);  
      $display("Running simulation for %d patterns:",numpat);
      //$monitor(".", test_se);
   end // initial begin
   
   //CHIP modul
   //chip_top U_CHIP_TOP(/*AUTOINST*/);



  chip U_CHIP(/*AUTOINST*/
              // Outputs
              .bist_done                (bist_done),
              .conf_scan_out            (conf_scan_out),
              .scan_inputs              (scan_inputs[NUM_SCAN_CHAINS-1:0]),
              .scan_en                  (scan_en),
              .ext_scan_out             (ext_scan_out[NUM_EXT_SCAN_CHAINS-1:0]),
              .sys_clk                  (sys_clk),
              // Inputs
              .rst_n                    (rst_n),
              .ext_clk                  (ext_clk),
              .bist_start               (bist_start),
              .ext_test_en              (ext_test_en),
              .ext_scan_en              (ext_scan_en),
              .ext_scan_in              (ext_scan_in[NUM_EXT_SCAN_CHAINS-1:0]),
              .conf_scan_en             (conf_scan_en),
              .conf_scan_in             (conf_scan_in),
              .scan_outputs             (scan_outputs[NUM_SCAN_CHAINS-1:0]),
              .osc_clk                  (osc_clk));

  osc U_OSC(/*AUTOINST*/
            // Outputs
            .osc_clk                    (osc_clk));

  
   spi_co_processor U_SPI_CO_PROCESSOR(/*AUTOINST*/
                                       // Outputs
                                       .debug           (debug[2:0]),
                                       .sdo             (sdo),
                                       .test_so1        (test_so1),
                                       .test_so2        (test_so2),
                                       .test_so3        (test_so3),
                                       .test_so4        (test_so4),
                                       .test_so5        (test_so5),
                                       .test_so6        (test_so6),
                                       .test_so7        (test_so7),
                                       .test_so8        (test_so8),
                                       .test_so9        (test_so9),
                                       .test_so10       (test_so10),
                                       .test_so11       (test_so11),
                                       .test_so12       (test_so12),
                                       .test_so13       (test_so13),
                                       .test_so15       (test_so15),
                                       .test_so16       (test_so16),
                                       // Inputs
                                       .rst_n           (rst_n),
                                       .clk             (clk),
                                       .sdi             (sdi),
                                       .ss_n            (ss_n),
                                       .sclk            (sclk),
                                       .test_se         (test_se),
                                       .test_si1        (test_si1),
                                       .test_si2        (test_si2),
                                       .test_si3        (test_si3),
                                       .test_si4        (test_si4),
                                       .test_si5        (test_si5),
                                       .test_si6        (test_si6),
                                       .test_si7        (test_si7),
                                       .test_si8        (test_si8),
                                       .test_si9        (test_si9),
                                       .test_si10       (test_si10),
                                       .test_si11       (test_si11),
                                       .test_si12       (test_si12),
                                       .test_si13       (test_si13),
                                       .test_si14       (test_si14),
                                       .test_si15       (test_si15),
                                       .test_si16       (test_si16));
  

  
  always @ (posedge clk)
    begin
      if (test_se)
        begin
          //test vectors
          chain_in[0] <= {chain_in[0][LEN_SCAN_CHAINS-2:0],test_si1};
          chain_in[1] <= {chain_in[1][LEN_SCAN_CHAINS-2:0],test_si2};
          chain_in[2] <= {chain_in[2][LEN_SCAN_CHAINS-2:0],test_si3};
          chain_in[3] <= {chain_in[3][LEN_SCAN_CHAINS-2:0],test_si4};
          chain_in[4] <= {chain_in[4][LEN_SCAN_CHAINS-2:0],test_si5};
          chain_in[5] <= {chain_in[5][LEN_SCAN_CHAINS-2:0],test_si6};
          chain_in[6] <= {chain_in[6][LEN_SCAN_CHAINS-2:0],test_si7};
          chain_in[7] <= {chain_in[7][LEN_SCAN_CHAINS-2:0],test_si8};
          chain_in[8] <= {chain_in[8][LEN_SCAN_CHAINS-2:0],test_si9};
          chain_in[9] <= {chain_in[9][LEN_SCAN_CHAINS-2:0],test_si10};
          chain_in[10] <= {chain_in[10][LEN_SCAN_CHAINS-2:0],test_si11};
          chain_in[11] <= {chain_in[11][LEN_SCAN_CHAINS-2:0],test_si12};
          chain_in[12] <= {chain_in[12][LEN_SCAN_CHAINS-2:0],test_si13};
          chain_in[13] <= {chain_in[13][LEN_SCAN_CHAINS-2:0],test_si14};
          chain_in[14] <= {chain_in[14][LEN_SCAN_CHAINS-2:0],test_si15};
          chain_in[15] <= {chain_in[15][LEN_SCAN_CHAINS-2:0],test_si16};

          //test responses
          chain_out[0] <= {chain_out[0][SHIFT_BITS-2:0],test_so1};
          chain_out[1] <= {chain_out[1][SHIFT_BITS-2:0],test_so2};
          chain_out[2] <= {chain_out[2][SHIFT_BITS-2:0],test_so3};
          chain_out[3] <= {chain_out[3][SHIFT_BITS-2:0],test_so4};
          chain_out[4] <= {chain_out[4][SHIFT_BITS-2:0],test_so5};
          chain_out[5] <= {chain_out[5][SHIFT_BITS-2:0],test_so6};
          chain_out[6] <= {chain_out[6][SHIFT_BITS-2:0],test_so7};
          chain_out[7] <= {chain_out[7][SHIFT_BITS-2:0],test_so8};
          chain_out[8] <= {chain_out[8][SHIFT_BITS-2:0],test_so9};
          chain_out[9] <= {chain_out[9][SHIFT_BITS-2:0],test_so10};
          chain_out[10] <= {chain_out[10][SHIFT_BITS-2:0],test_so11};
          chain_out[11] <= {chain_out[11][SHIFT_BITS-2:0],test_so12};
          chain_out[12] <= {chain_out[12][SHIFT_BITS-2:0],test_so13};
          chain_out[13] <= {chain_out[13][SHIFT_BITS-2:0],test_so14};
          chain_out[14] <= {chain_out[14][SHIFT_BITS-2:0],test_so15};
          chain_out[15] <= {chain_out[15][SHIFT_BITS-2:0],test_so16};
          
        end
    end

    /*
    * Simulation time out
    *
    */
   initial begin
     fdscn = $fopen("./genpat/atpg_patterns_ser.scn", "wb");
     fdpis = $fopen("./genpat/atpg_patterns_ser.pis", "wb");
     fdpos = $fopen("./genpat/atpg_patterns_ser.pos", "wb");
     fdseq = $fopen("./genpat/atpg_patterns_ser.seq", "wb");
     patterncounter = 0;
     
     #1;
     @(negedge bist_done);
     #1000;
     
     $fclose(fdscn);
     $fclose(fdpis);
     $fclose(fdpos);
     $fclose(fdseq);
      $finish;
   end


  initial begin
    // scn file
    $fwrite(fdscn, "// Verilog scan readmem data written by testbench.v, written by Tor Erik\n");
    $fwrite(fdscn, "// DATE: \n");
    $fwrite(fdscn, "// Module tested: spi_co_processor\n");
    $fwrite(fdscn, "\n");

    // pis file
    $fwrite(fdpis, "// Verilog scan readmem data written by testbench.v, written by Tor Erik\n");
    $fwrite(fdpis, "// DATE: \n");
    $fwrite(fdpis, "// Module tested: spi_co_processor\n");
    $fwrite(fdpis, "\n");
        
    // pos file
    $fwrite(fdpos, "// Verilog scan readmem data written by testbench.v, written by Tor Erik\n");
    $fwrite(fdpos, "// DATE: \n");
    $fwrite(fdpos, "// Module tested: spi_co_processor\n");
    $fwrite(fdpos, "\n");

    // seq file
    $fwrite(fdseq, "// Verilog driver readmem data written by testbench.v, written by Tor Erik\n");
    $fwrite(fdseq, "// DATE: \n");
    $fwrite(fdseq, "// Module tested: spi_co_processor\n");
    $fwrite(fdseq, "\n");
    
  end
  
  //output pattern file generation
  always @ (negedge test_se)
    begin

      @(negedge clk);
      #0;
      
      if(bist_started)
        begin
          for(i=0; i<16; i=i+1)
            begin
              //this is the scan in data, into .scn file
              $fwrite(fdscn, "%7H\n", chain_in[i]);
            end
              
          if(patterncounter < 1) 
            begin
              //write out the expected output from simulation but mask all of it
              for(i=0; i<16; i=i+1)
                begin
                  $fwrite(fdscn, "%7H\n",  chain_out[i][SHIFT_BITS-1:SHIFT_BITS-LEN_SCAN_CHAINS]); //write scan output
                  if(i<10)
                    $fwrite(fdscn, "0000000\n");//write mask, longest scan chains
                  else
                    $fwrite(fdscn, "0000000\n");//write mask, shortest scan chains
                end
            end // if (patterncounter < 1)
          else
            begin
              //this is the scan out data, into .scn file
              if(patterncounter > 0) 
                begin
                  for(i=0; i<16; i=i+1)
                    begin
                      $fwrite(fdscn, "%7H\n",  chain_out[i][SHIFT_BITS-1:SHIFT_BITS-LEN_SCAN_CHAINS]); //write scan output
                      if(i<3)
                        $fwrite(fdscn, "FFFFFFF\n");//write mask, longest scan chains
                      else
                        $fwrite(fdscn, "FFFFFFE\n");//write mask, shortest scan chains
                    end
                end
            end // else: !if(patterncounter < 1)    

          //PIs and POs
          if(patterncounter < (numpat))
            begin
              //primary inputs
              $fwrite(fdpis, "%b\n", pis);
              if(fmodel == TRAN)
                $fwrite(fdpis, "%b\n", pis); //write the same PI again, we dont change PI
              if(fmodel == PATH)
                $fwrite(fdpis, "%b\n", pis); //write the same PI again, we dont change PI
              //primary outputs
              if(fmodel == TRAN)
                @(negedge clk);
              if(fmodel == PATH)
                @(negedge clk);
              $fwrite(fdpos, "%b\n", pos);
              $fwrite(fdpos, "%b\n", 19'h7FFFF);
            end

          //driver readmem into .seq file
          if(patterncounter < (numpat-1))
            begin
              $fwrite(fdseq, "0  // pattern %d\n",patterncounter);
              $fwrite(fdseq, "7\n");
              $fwrite(fdseq, "8\n");
              $fwrite(fdseq, "1D\n");
              if(fmodel == TRAN)
                begin
                  $fwrite(fdseq, "4\n");
                  $fwrite(fdseq, "6\n");
                  $fwrite(fdseq, "18\n");
                end
              if(fmodel == PATH)
                begin
                  $fwrite(fdseq, "4\n");
                  $fwrite(fdseq, "6\n");
                  $fwrite(fdseq, "18\n");
                end
              $fwrite(fdseq, "4\n");
              $fwrite(fdseq, "5\n");
              $fwrite(fdseq, "18\n");
            end // else: !if(patterncounter < 1)
          else
            if (patterncounter == (numpat-1))
              begin
                $fwrite(fdseq, "0  // pattern %d\n",patterncounter);
                $fwrite(fdseq, "7\n");
                $fwrite(fdseq, "8\n");
                $fwrite(fdseq, "1D\n");
                if(fmodel == TRAN)
                begin
                  $fwrite(fdseq, "4\n");
                  $fwrite(fdseq, "6\n");
                  $fwrite(fdseq, "18\n");
                end
              if(fmodel == PATH)
                begin
                  $fwrite(fdseq, "4\n");
                  $fwrite(fdseq, "6\n");
                  $fwrite(fdseq, "18\n");
                end
                $fwrite(fdseq, "4\n");
                $fwrite(fdseq, "5\n");
                $fwrite(fdseq, "18\n");  //18 - tetramax places a capture_rst_n here, but since
                $fwrite(fdseq, "1\n");   // bist cant do reset, we use regular capture_clk
                $fwrite(fdseq, "7\n");
                $fwrite(fdseq, "8\n");
                $fwrite(fdseq, "1D\n");
                $fwrite(fdseq, "2\n");
              end // if (patterncounter == (numpat-1))

          //inc pattern counter
          patterncounter = patterncounter + 1;
          if((patterncounter%10) == 0)
            $display("%d of %d", patterncounter, numpat);

        end // if (bist_started)
      else
        bist_started = 1;          
    end // always @ (negedge test_se)
  
  
endmodule // testbench

/*
 Local Variables:
 verilog-library-directories:(
 "."
 "../../rtl/module/chip/custom/verilog/"
 "../../synth/dut_atmel/"
 "../../rtl/module/chip/verilog/")
 End:
*/
