//****************************************************************************
//                 PROPRIETARY INFORMATION OF ATMEL CORPORATION
//
//                     
//                       Copyright Atmel Corporation
//
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the LICENSE which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than to the
//           EMPLOYEES of the Licensee, as the term is defined in the
//        LICENSE, is strictly forbidden without a prior written consent
//                            of Atmel Corporation.
//****************************************************************************
//****************************************************************************
// 
//  RTL IMPLEMENTATION, Delay-fault BIST in low-power CMOS devices 
//
//  Filename        :  testbench.v
//  Revision        :  $Revision: 1.1 $
//  Checkin Date    :  $Date: 2008/03/09 12:29:59 $
//  Last checkin by :  $Author: teleistad $
//  SVN State       :  $State: Exp $
//
//  Project number  :  ? , Master Thesis
//  Module name     :  <testbench>
//  Description     :  <test the dut, spi_co_processor>
//
//  Author          :  <Tor Erik Leistad>
//  Date            :  <02.2008>
//
//
//****************************************************************************


module testbench();

   `include "spi_co_processor_params.v"

   parameter clock_period = 2;
   parameter spiclock_period = clock_period * 100;
   
   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   // End of automatics

   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire [2:0]		debug;			// From U_SPI_CO_PROCESSOR of spi_co_processor.v
   wire			sdo;			// From U_SPI_CO_PROCESSOR of spi_co_processor.v
   // End of automatics

   
   //regs for modul inputs
   //reg [:] name;
   reg 			ss_n;
   reg 			sdi;
   reg 			sclk;
   
   //wires for modul inputs

   //other regs
   reg 			clk;
   reg 			rst_n;
   reg 			sclk_run;

   reg [INSTRBITS-1:0] 	cmd;
   reg [CPUBITS-1:0] 	data;

   /*
    * Task for sending cmd
    * 
    */
   integer 		i;
   task sendcmd;
      input [6:0] instr;
      input [2:0] addrd;
      input [2:0] addra;
      input [2:0] addrb;
      begin
	 cmd <= {instr,addrd,addra,addrb};
	 ss_n <= 1'b0;
	 #(spiclock_period/2);
	 sclk_run <= 'b1;
	 
	 for(i = 0; i<=15; i=i+1)
	   begin
	      sdi <= cmd[15];
	      cmd <= {cmd[14:0],sdo};
	      @(negedge sclk);
	   end
	 sclk_run <= 'b0;
	 #(spiclock_period/2);
	 ss_n <= 'b1;
      end
   endtask // sendcmd


   /*
    * Task for sending cmd and data
    * also used to receive data
    */
   task sendcmddata; //(instr,addrd,addra,addrb,data);
      input [6:0] instr;
      input [2:0] addrd;
      input [2:0] addra;
      input [2:0] addrb;
      input [31:0] in;
      begin
	 cmd <= {instr,addrd,addra,addrb};
	 data <= in;
	 ss_n <= 1'b0;
	 #(spiclock_period/2);
	 sclk_run <= 'b1;
	 
	 for(i = 0; i<=15; i=i+1)
	   begin
	      sdi <= cmd[15];
	      cmd <= {cmd[14:0],sdo};
	      @(negedge sclk);
	   end
	 for(i = 0; i<=(CPUBITS-1); i=i+1)
	   begin
	      //sdi <= data[CPUBITS-1-i];
	      sdi <= data[CPUBITS-1];
	      data <= {data[CPUBITS-2:0],sdo};
	      @(negedge sclk);
	   end
	 sclk_run <= 'b0;
	 #(spiclock_period/2);
	 ss_n <= 'b1;
      end
   endtask // sendcmd


   /*
    * Generate SPI clock
    *
    */
   initial begin
      sclk = 'b0;
      sclk_run = 'b0;
   end
   always #(spiclock_period/2) if(sclk_run == 'b1 ) sclk = !sclk;
   
   
 
   /*
    * Generate clock
    *
    */
   initial begin
      clk = 'b1;
   end
   always #(clock_period/2) clk = !clk;
   
   /*
    * Generate reset
    *
    */
   initial begin
      rst_n = 'b0;
      #100;
      rst_n = 'b1;
   end
     
   /*
    * VCD logging
    *
    */
   initial begin
      $display("Enable logging.");
      #1; //this tick contains just useless stuff...
      $vcdpluson(0); //use with design compiler
      //$dumpfile("vsim.vcd"); 
      //$dumpvars(5,U_SPI_CO_PROCESSOR);
      $dumpvars;
      
   end

   /*
    * Simulation time out
    *
    */
   initial begin
      #50000;
      $finish;
   end

   /*
    * TEST sequencs
    *
    */
   initial begin
      
      ss_n <= 1'b1;
      sdi <= 1'b0;
      
      //wait for reset at #100
      #200;
      sendcmddata(WRITE,R0,RX,RX,32'h00000001);
      #1000;
      sendcmddata(WRITE,R1,RX,RX,32'h00000001);
      #1000;
      sendcmd(ADD,R2,R1,R0);
      #1000;
      sendcmddata(READ,RX,R2,RX,32'hAAAA5555);
      #1000;
      
      
      
      //$display("LFSR output:");
      //$monitor("%t: [%x]",$time,`TB.lfsr_out);
   end // initial begin
   
   //DUT/CUT moodul
   spi_co_processor U_SPI_CO_PROCESSOR(/*AUTOINST*/
				       // Outputs
				       .sdo(sdo),
				       .debug(debug[2:0]),
				       // Inputs
				       .rst_n(rst_n),
				       .clk(clk),
				       .sdi(sdi),
				       .ss_n(ss_n),
				       .sclk(sclk));
   


endmodule // testbench

/*
 Local Variables:
 verilog-library-directories:(
 "."
 "../../rtl/module/dut/custom/verilog/")
 End:
*/
