// Simple module testbench
//
//

module testbench();

   //`include "_params.v"

   parameter clock_period = 20;

   /*AUTOREG*/
   // Beginning of automatic regs (for this module's undeclared outputs)
   // End of automatics

   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire [7:0]		lfsr_out;		// From U_LFSR of lfsr.v
   wire			lfsr_so;		// From U_LFSR of lfsr.v
   // End of automatics

   
   //regs for modul inputs
   //reg [:] name;
   reg 			lfsr_en;
   reg 			lfsr_si;
   reg 			lfsr_load;
        
   //wires for modul inputs
   wire 		lfsr_clk;
   wire 		lfsr_rst_n;

   //other regs
   reg 			clk;
   reg 			rst_n;

   assign lfsr_clk = clk;
   assign lfsr_rst_n = rst_n;

   
   /*
    * Generate clock
    *
    */
   initial begin
      clk = 'b1;
   end
   always #(clock_period/2) clk = !clk;
   
   /*
    * Generate reset
    *
    */
   initial begin
      rst_n = 'b0;
      #100;
      rst_n = 'b1;
   end
     
   /*
    * VCD logging
    *
    */
   initial begin
      $display("Enable logging.");
      #1; //this tick contains just useless stuff...
      //$vcdpluson(0); //use with design compiler
      $dumpfile("lfsr.vcd"); //use with iverilog
      $dumpvars(0,U_LFSR); //use with iverilog
   end

   /*
    * Simulation time out
    *
    */
   initial begin
      #10000;
      $finish;
   end

   /*
    * TEST sequencs
    *
    */
   initial begin

      lfsr_load <= 'b0;
      lfsr_en <= 'b1;
      lfsr_si <= 'b0;
      
      //wait for reset at #100
      #200;

      //$display("LFSR output:");
      //$monitor("%t: [%x]",$time,`TB.lfsr_out);
   end // initial begin
   
   //DUT/CUT moodul
   lfsr U_LFSR(/*AUTOINST*/
	       // Outputs
	       .lfsr_so			(lfsr_so),
	       .lfsr_out		(lfsr_out[7:0]),
	       // Inputs
	       .lfsr_clk		(lfsr_clk),
	       .lfsr_rst_n		(lfsr_rst_n),
	       .lfsr_en			(lfsr_en),
	       .lfsr_load		(lfsr_load),
	       .lfsr_si			(lfsr_si));
   


endmodule // testbench

/*
 Local Variables:
 verilog-library-directories:(
 "."
 "../../rtl/module/lfsr/custom/verilog/")
 End:
*/
